@@ -433,15 +433,25 @@ def modified_func_node(func_node, **modifications) -> FuncNode:
     return FuncNode(**dict(original_func_node_kwargs, **modifications))
 
 
+from meshed.util import partialx
+
+# TODO: doctests
 def partialized_funcnodes(func_nodes, **keyword_defaults):
     for func_node in func_nodes:
         if argnames_to_be_bound := set(keyword_defaults).intersection(
             func_node.sig.names
         ):
             bindings = dict(extract_items(keyword_defaults, argnames_to_be_bound))
+            # partialize the func and move defaulted params to the end
+            partialized_func = partialx(
+                func_node.func, **bindings, _allow_reordering=True
+            )
+            # get rid of kinds  # TODO: This is a bit extreme -- consider gentler touch
+            nice_kinds_sig = Sig(partialized_func).ch_kinds_to_position_or_keyword()
+            nice_kinds_partialized_func = nice_kinds_sig(partialized_func)
             yield modified_func_node(
-                func_node, func=partial(func_node.func, **bindings)
-            )  # TODO: Try without partial?
+                func_node, func=nice_kinds_partialized_func
+            )  # TODO: A better way without partial?
         else:
             yield func_node
 
@@ -746,7 +756,7 @@ class DAG:
         >>> isinstance(new_dag, DAG)  # it's a dag (not just a partialized callable!)
         True
         >>> str(signature(new_dag))
-        '(a, b, *, c=3, d=4)'
+        '(a, b, c=3, d=4)'
         >>> new_dag(1, 2)  # same as dag(c=3, a=1, b=2, d=4), so:
         9
         """
